# -*- encoding: utf-8 -*-

module Stomp

  # Define the gem version.
  module Version  #:nodoc: all
    MAJOR = 1
    MINOR = 4
    PATCH = "10"
    MODLEVEL = ""
    STRING = "#{MAJOR}.#{MINOR}.#{PATCH}#{MODLEVEL}"
    # Alternate version string
    Version = STRING
  end
end
